@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2022  Jean-Philippe Lang
+# Copyright (C) 2006-2023  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -273,15 +273,15 @@ module Redmine
       # %m1%, %m2%... => capture groups matches of the custom field regexp if defined
       def url_from_pattern(custom_field, value, customized)
         url = custom_field.url_pattern.to_s.dup
-        url.gsub!('%value%') {Addressable::URI.encode value.to_s}
-        url.gsub!('%id%') {Addressable::URI.encode customized.id.to_s}
+        url.gsub!('%value%') {Addressable::URI.encode_component value.to_s}
+        url.gsub!('%id%') {Addressable::URI.encode_component customized.id.to_s}
         url.gsub!('%project_id%') do
-          Addressable::URI.encode(
+          Addressable::URI.encode_component(
             (customized.respond_to?(:project) ? customized.project.try(:id) : nil).to_s
           )
         end
         url.gsub!('%project_identifier%') do
-          Addressable::URI.encode(
+          Addressable::URI.encode_component(
             (customized.respond_to?(:project) ? customized.project.try(:identifier) : nil).to_s
           )
         end
@@ -289,7 +289,7 @@ module Redmine
           url.gsub!(%r{%m(\d+)%}) do
             m = $1.to_i
             if matches ||= value.to_s.match(Regexp.new(custom_field.regexp))
-              Addressable::URI.encode matches[m].to_s
+              Addressable::URI.encode_component matches[m].to_s
             end
           end
         end
